/*
 *				**** CSV XML ****
 *
 * This program reads a CSV file and creates an XML file
 *
 * The import file should have field names in the first line
 * which will be used as the XML field names
 *
 * The output routine will be provided a record name
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "wrpcsv.h"

#define MAXFLD 100

void process_csv(FILE *fp, char *fname, char *cname, char *rname, char aflag, FILE  *ffp);
char *xattr(char *s);

char *Prog=NULL;		/* program name */
int debugsw=0;

int main(int argc, char *argv[])
{
	char buf[BUFSIZ], *p, *q;
   char *rname=NULL, *fname=NULL, *cname=NULL;
   char aflag = 'a';		/* flag to convert fields into trees or attributes */
   FILE *fp;
   FILE *ffp = NULL;		/* file pointer for field names (optional) */

   if ((p = strrchr(argv[0],'/')) == NULL) p = strrchr(argv[0],'\\');
   if (p == NULL) p = argv[0];
   if ((q=strstr(p, ".exe"))==NULL) q = strstr(p, ".EXE");
   if (q != NULL) *q = '\0';
   Prog = strdup(p);		/* get program name */
   argc--; argv++;		/* shift program name out */

   if (argc > 0)
   {
   	while (argc > 0)
	   { /* process arguments */
			if (argv[0][0]=='-') switch(argv[0][1])
         { /* process option */
         case 'a':	/* attribute type (t=>tree, a=>attribute) */
         	if (argv[0][2]== '0')
            { /* flag value is next argument */
            	argc--; argv++;
               if (argc > 0) aflag = argv[0][0]; /* single character */
            }
            else
            { /* flag value follows flag */
            	aflag = argv[0][2];
            }
            if (debugsw) printf("AFlag >>%c<<\n", aflag);
            break;
         case 'r': /* record name */
         	if (argv[0][2]=='\0')
            { /* record name is next argument */
            	argc--; argv++;
               if (argc > 0) rname = strdup(argv[0]);
            }
            else
            { /* record name is rest of this argument */
            	rname = strdup(argv[0]+2);
            }
            if (debugsw) printf("RecordName >>%s<<\n", rname);
            break;
         case 'c': /* collection name */
         	if (argv[0][2]=='\0')
            { /* collection name is next argument */
            	argc--; argv++;
               if (argc > 0) cname = strdup(argv[0]);
            }
            else
            { /* collection name is rest of this argument */
            	cname = strdup(argv[0]+2);
            }
            if (debugsw) printf("CollectionName >>%s<<\n", cname);
            break;
         case 'f': /* field names file */
         	if (argv[0][2] == '\0')
            { /* file name is  next argument */
            	argc--; argv++;
               p = argv[0];
            }
            else
            {
            	p = argv[0] + 2;
            }
            if (debugsw) printf("FieldNameFile >>%s<<\n", p);
            if ((ffp = fopen(p, "r")) == NULL)
            {
            	fprintf(stderr, "Can't open file with field names (%s)\n", p);
            }
         	break;
         case 'o': /* output file name */
         	if (argv[0][2]=='\0')
            { /* file name is next argument */
            	argc--; argv++;
               if (argc > 0) fname = strdup(argv[0]);
            }
            else
            { /* file name is rest of this argument */
            	fname = strdup(argv[0]+2);
            }
            if (debugsw) printf("OutputFile >>%s<<\n", fname);
            break;
         case 'd': /* debug switch */
         	debugsw++;
            printf("Debugsw turned on\n");
            break;
			default:
         	fprintf(stderr, "Invalid option (%s)\n", argv[0]);
         }
         else
         { /* file name */
         	if (debugsw) printf("File to Process >>%s<<\n", argv[0]);
         	if ((fp = fopen(argv[0],"r"))==NULL)
            {
            	fprintf(stderr, "Can't open input file (%s)\n", argv[0]);
            }
            else
            {
         		if (rname != NULL && cname != NULL && fname != NULL)
               {
               	process_csv(fp, fname, cname, rname, aflag, ffp);
               }
               else
               {
               	fprintf(stderr, "Missing options for file (%s)\n", argv[0]);
               }
               if (debugsw) printf("Closing input file\n");
               fclose(fp);
            }
            if (debugsw) printf("Free rname\n");
            if (rname != NULL) free(rname);
            if (debugsw) printf("Free cname\n");
            if (cname != NULL) free(cname);
            if (debugsw) printf("Free fname\n");
            if (fname != NULL) free(fname);
            rname=NULL;	/* turn off record name */
            cname=NULL;
            fname=NULL;
            if (ffp != NULL)
            { /* turn off field names file */
            	if (debugsw) printf("Closing Field names file\n");
            	fclose(ffp);
               ffp = NULL;
            }
         }
	   	argc--; argv++;
	   } /* end loop thru arguments */
   }
   else
   {
   	while (1)
      {
			printf("Enter input file, or ENTER to quit: ");
         gets(buf);
         if (*buf == '\0') break;
         if ((fp = fopen(buf, "r")) == NULL)
         {
         	printf("Can't open input file (%s)\n", buf);
            continue;
         }
         printf("Enter record name: ");
         gets(buf);
         rname = strdup(buf);
         printf("Enter collection name: ");
         gets(buf);
         cname = strdup(buf);
         printf("Enter output file name: ");
         gets(buf);
         fname = strdup(buf);
         printf("Enter field type (t=>tree, a=>attribute) [%c]: ", aflag);
         gets(buf);
         if (*buf != '\0') aflag = *buf;
         printf("Enter file with field names (or ENTER if in first line): ");
         gets(buf);
         if (*buf != '\0')
         { /* using field name file */
         	if ((ffp = fopen(buf, "r")) == NULL)
            {
            	fprintf(stderr, "Can't open field names file\n");
            }
         }
         process_csv(fp, fname, cname, rname, aflag, ffp);
         free(rname);
         free(cname);
         free(fname);
         fclose(fp);
      }
   }
	printf("Press ENTER to terminate program: ");
   gets(buf);

   return 0;
}

void process_csv(FILE *fp, char *fname, char *cname, char *rname, char aflag, FILE *ffp)
{
	int i, csv, mfld, ret;
   char *flds[MAXFLD], *p;
   FILE *xfp;		/* output XML file */

   if (debugsw) printf("Opening output file (%s)\n", fname);
   if ((xfp = fopen(fname, "w")) == NULL)
   {
   	fprintf(stderr, "Can't open output file (%s)\n", fname);
      return;
   }
   if ((csv = wrpCsvCreate())<0)
   {
   	fprintf(stderr, "Can't create CSV!\n");
      return;
   }
   /* get field names from separate file or first line */
   if (ffp != NULL)
   {
   	if (debugsw) printf("Reading field names from file\n");
   	ret = wrpCsvReadLine(csv, ffp);
   }
   else
   {
   	if (debugsw) printf("Reading field names from first line\n");
   	ret = wrpCsvReadLine(csv, fp);
   }
   if (ret != WRPCSV_OK)
   {
   	fprintf(stderr, "Problems reading field names!\n");
      return;
   }
   mfld = wrpCsvGetNumFields(csv);	/* how many fields */
   if (debugsw) printf("%d fields found\n", mfld);
   /* fill in field names */
   for (i=0; i < mfld; i++)
   {
   	p = wrpCsvGetField(csv, i);
      if (p != NULL) flds[i] = strdup(p);
      else flds[i] = NULL;
      if (debugsw) printf("Fld[%d] >>%s<<\n", i, p);
   }

   fprintf(xfp, "<%s>\n", cname);	/* start collection */

   /* now loop thru file and create records for each line */
   while ((ret=wrpCsvReadLine(csv, fp)) == WRPCSV_OK)
   {
   	if (debugsw) printf("CSV Line read\n");
   	if (aflag == 't')
      { /* write fields as subtrees */
	      if (debugsw) printf("Writing attributes as subtrees\n");
	   	fprintf(xfp, "\n<%s>\n", rname);
	   	for (i=0; i < mfld; i++)
	      { /* write out each field */
	      	p = wrpCsvGetField(csv, i);
	         if (p == NULL || *p == '\0') p = "[NULL]";
            if (debugsw) printf("Fld[%d] >>%s<<, Val >>%s<<\n", i, flds[i], p);
	      	fprintf(xfp, "<%s>%s</%s>\n", flds[i], p, flds[i]);
	      }
	   	fprintf(xfp, "</%s>\n", rname);	/* terminate record */
      }
      else
      { /* write fields as attributes */
      	fprintf(xfp, "\n<%s\n", rname);
         for (i=0; i < mfld; i++)
         { /* write each field as an attribute */
         	p = wrpCsvGetField(csv, i);	/* get value */
            p = xattr(p);	/* convert for attribute */
            if (debugsw) printf("Fld[%d] >>%s<<, Val >>%s<<\n", i, flds[i], p);
         	fprintf(xfp, "  %s=\"%s\"\n", flds[i], p);
         }
         fprintf(xfp, "/>\n");
      }
      fflush(xfp);
   }

   fprintf(xfp, "\n</%s>\n", cname);
   if (debugsw) printf("Record written\n");
}

char *xattr(char *s)
{
	static char ret[BUFSIZ];
	char *p;

   if (s == NULL)
   {
   	*ret = '\0';
      return ret;
   }
   /* something passed - replace double quotes with single */
   p = ret;
   for (p=ret; *s != '\0'; s++)
   {
   	*p++ = (*s == '"') ? '\'' : *s;
   }
   *p = '\0';	/* terminate string */

   return ret;
}


